<?xml version="1.0" encoding="UTF-8"?>

<!--
    Document   : Carnot2FXML.xsl
    Created on : 17 february 2022, 14:29
    Author     : Stéphane Bergeon
    Description:
        Transformation from ExecutableRecipeElementONly to ExecutableRecipe
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                version="1.0">
    <xsl:output method="xml" indent="no" encoding="UTF-8"/>

  
    <xsl:template match="/">
        <xsl:apply-templates select="@*|node()"/>
    </xsl:template>

    <xsl:template match="executable_recipe" priority="2">
        <executable_recipe>
            <xsl:attribute name="xsi:noNamespaceSchemaLocation">ExecutableRecipe.xsd</xsl:attribute>   
            <xsl:apply-templates select="sphere"/>
            <xsl:apply-templates select="frame_types"/>
            <xsl:apply-templates select="recipe"/>
            <xsl:apply-templates select="executable"/>
            <xsl:apply-templates select="inputs"/>
            <xsl:apply-templates select="outputs"/>
            <xsl:apply-templates select="parameters"/>
        </executable_recipe>  
    </xsl:template>
    
    <xsl:template match="@*|node()">
        <xsl:copy>
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="include" priority="2">
        <xsl:copy>
            <xsl:attribute name="value">
                <xsl:value-of select="value"/>
            </xsl:attribute>
            <xsl:attribute name="regex">
                <xsl:value-of select="regex"/>
            </xsl:attribute>    
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="include[value='']" priority="2">
        <xsl:copy>
            <xsl:attribute name="regex">
                <xsl:value-of select="regex"/>
            </xsl:attribute>    
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="include[regex='']" priority="2">
        <xsl:copy>
            <xsl:attribute name="value">
                <xsl:value-of select="value"/>
            </xsl:attribute>
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="sphere" priority="2">
        <xsl:copy>
            <xsl:attribute name="pipeline">
                <xsl:value-of select="pipeline"/>
            </xsl:attribute>
            <xsl:attribute name="user">
                <xsl:value-of select="user"/>
            </xsl:attribute>    
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="recipe/description" priority="2">
        <xsl:copy>
            <xsl:attribute name="short">
                <xsl:value-of select="short"/>
            </xsl:attribute>
            <xsl:attribute name="full">
                <xsl:value-of select="full"/>
            </xsl:attribute>    
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="input_frame" >
        <xsl:copy>
            <xsl:attribute name="min">
                <xsl:value-of select="min"/>
            </xsl:attribute>
            <xsl:attribute name="max">
                <xsl:value-of select="max"/>
            </xsl:attribute>    
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="author" priority="2">
        <xsl:copy>
            <xsl:attribute name="email">
                <xsl:value-of select="email"/>
            </xsl:attribute>    
            <xsl:attribute name="name">
                <xsl:value-of select="name"/>
            </xsl:attribute>
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="frame_types/frame_type" priority="2">
        <xsl:copy>
            <xsl:attribute name="name">
                <xsl:value-of select="name"/>
            </xsl:attribute>
            <xsl:attribute name="description">
                <xsl:value-of select="description"/>
            </xsl:attribute>    
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="recipe" priority="2">
        <xsl:copy>
            <xsl:attribute name="name">
                <xsl:value-of select="name"/>
            </xsl:attribute>
            <xsl:attribute name="instrument">
                <xsl:value-of select="instrument"/>
            </xsl:attribute>    
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="inputs" priority="2">
        <xsl:copy>
            <xsl:attribute name="use_sof">
                <xsl:value-of select="use_sof"/>
            </xsl:attribute>
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="outputs" priority="2">
        <xsl:copy>
            <xsl:attribute name="use_filename_parameter">
                <xsl:value-of select="use_filename_parameter"/>
            </xsl:attribute>
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="parameter" priority="2">
        <xsl:copy>
            <xsl:attribute name="name">
                <xsl:value-of select="name"/>
            </xsl:attribute>
            <xsl:attribute name="optional">
                <xsl:value-of select="optional"/>
            </xsl:attribute>
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="parameters" priority="2">
        <xsl:copy>
            <xsl:attribute name="prefix">
                <xsl:value-of select="prefix"/>
            </xsl:attribute>
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="output_frame | extra_output_frame" priority="2">
        <xsl:copy>
            <xsl:attribute name="format">
                <xsl:value-of select="format"/>
            </xsl:attribute>
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="output_frames | extra_output_frames" priority="2">
        <xsl:apply-templates select="@*|node()"/>
    </xsl:template>

    <xsl:template match="*[name]" priority="1">
        <xsl:copy>
            <xsl:attribute name="name">
                <xsl:value-of select="name"/>
            </xsl:attribute>
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>  
    </xsl:template>

    <xsl:template match="*[not(*)]" priority="1">
        <xsl:copy>
            <xsl:attribute name="value">
                <xsl:value-of select="."/>
            </xsl:attribute>
        </xsl:copy>  
    </xsl:template>
    
    <xsl:template match="frame_types[not(*)]" priority="2">
        <xsl:copy/> 
    </xsl:template>

    <xsl:template match="includes[not(*)]" priority="2">
        <xsl:copy/> 
    </xsl:template>

    <xsl:template match="name | recipe/instrument | use_filename_parameter | use_sof | input_frame/min | input_frame/max | prefix | optional | format" priority="2"/>

</xsl:stylesheet>